@@ -1,10 +1,20 @@
 from pathlib import Path
 
-from .utils.gui import run  # noqa
+# this must be the first import for auto-canvas detection
+from .utils import loop  # noqa
 from .graphics import *
 from .graphics.selectors import *
+from .graphics.utils import pause_events
 from .legends import *
-from .layouts import Figure
+from .tools import *
+
+from .layouts import IMGUI
+
+if IMGUI:
+    # default to imgui figure if imgui_bundle is installed
+    from .layouts import ImguiFigure as Figure
+else:
+    from .layouts import Figure
 
 from .widgets import ImageWidget
 from .utils import config, enumerate_adapters, select_adapter, print_wgpu_report
@@ -14,7 +24,9 @@ with open(Path(__file__).parent.joinpath("VERSION"), "r") as f:
     __version__ = f.read().split("\n")[0]
 
 if len(enumerate_adapters()) < 1:
-    raise IndexError(
+    from warnings import warn
+
+    warn(
         f"WGPU could not enumerate any adapters, fastplotlib will not work.\n"
         f"This is caused by one of the following:\n"
         f"1. You do not have a hardware GPU installed and you do not have "
@@ -25,5 +37,6 @@ if len(enumerate_adapters()) < 1:
         f"common in cloud computing environments.\n"
         f"These two links can help you troubleshoot:\n"
         f"https://wgpu-py.readthedocs.io/en/stable/start.html#platform-requirements\n"
-        f"https://fastplotlib.readthedocs.io/en/latest/user_guide/gpu.html\n"
+        f"https://fastplotlib.readthedocs.io/en/latest/user_guide/gpu.html\n",
+        RuntimeWarning,
     )
