@@ -7,6 +7,13 @@ import numpy as np
 import pylinalg as la
 from wgpu.gui.base import log_exception
 
+try:
+    from imgui_bundle import imgui
+except ImportError:
+    IMGUI = False
+else:
+    IMGUI = True
+
 import pygfx
 
 from ._features import (
@@ -17,6 +24,7 @@ from ._features import (
     Rotation,
     Visible,
 )
+from ._axes import Axes
 
 HexStr: TypeAlias = str
 
@@ -42,7 +50,7 @@ PYGFX_EVENTS = [
 
 
 class Graphic:
-    _features = {}
+    _features: set[str] = {}
 
     def __init_subclass__(cls, **kwargs):
         # set the type of the graphic in lower case like "image", "line_collection", etc.
@@ -114,6 +122,10 @@ class Graphic:
         self._visible = Visible(visible)
         self._block_events = False
 
+        self._axes: Axes = None
+
+        self._right_click_menu = None
+
     @property
     def supported_events(self) -> tuple[str]:
         """events supported by this graphic"""
@@ -177,7 +189,7 @@ class Graphic:
     def world_object(self) -> pygfx.WorldObject:
         """Associated pygfx WorldObject. Always returns a proxy, real object cannot be accessed directly."""
         # We use weakref to simplify garbage collection
-        return weakref.proxy(WORLD_OBJECTS[self._fpl_address])
+        return weakref.proxy(WORLD_OBJECTS[hex(id(self))])
 
     def _set_world_object(self, wo: pygfx.WorldObject):
         WORLD_OBJECTS[self._fpl_address] = wo
@@ -208,22 +220,18 @@ class Graphic:
 
     def add_event_handler(self, *args):
         """
-        Register an event handler.
+        Register an event handler. Can also be used as a decorator.
 
         Parameters
         ----------
         callback: callable, the first argument
             Event handler, must accept a single event  argument
-        *types: list of strings
-            A list of event types, ex: "click", "data", "colors", "pointer_down"
 
-        For the available renderer event types, see
-        https://jupyter-rfb.readthedocs.io/en/stable/events.html
+        *types: strings
+            event types, ex: "click", "data", "colors", "pointer_down"
 
-        All feature support events, i.e. ``graphic.features`` will give a set of
-        all features that are evented
-
-        Can also be used as a decorator.
+            ``supported_events`` will return a tuple of all event type strings that this graphic supports.
+            See the user guide in the documentation for more information on events.
 
         Example
         -------
@@ -300,21 +308,38 @@ class Graphic:
             # for feature events
             event._target = self.world_object
 
-        if isinstance(event, pygfx.PointerEvent):
-            # map from screen to world space and data space
-            world_xy = self._plot_area.map_screen_to_world(event)
-
-            # subtract offset to map to data
-            data_xy = world_xy - self.offset
-
-            # append attributes
-            event.x_world, event.y_world = world_xy[:2]
-            event.x_data, event.y_data = data_xy[:2]
-
         with log_exception(f"Error during handling {event.type} event"):
             callback(event)
 
     def remove_event_handler(self, callback, *types):
+        """
+        remove a registered event handler
+
+        Parameters
+        ----------
+        callback: callable
+            event handler that has been added
+
+        *types: strings
+            event types that were registered with the given callback
+
+        Example
+        -------
+
+        .. code-block:: py
+
+            # define event handler
+            def my_handler(event):
+                print(event)
+
+            # add event handler
+            graphic.add_event_handler(my_handler, "pointer_up", "pointer_down")
+
+            # remove event handler
+            graphic.remove_event_handler(my_handler, "pointer_up", "pointer_down")
+
+        """
+
         # remove from our record first
         for t in types:
             for wrapper_map in self._event_handler_wrappers[t]:
@@ -331,8 +356,6 @@ class Graphic:
             self._event_handlers[t].remove(callback)
             # remove callback wrapper from world object if pygfx event
             if t in PYGFX_EVENTS:
-                print("pygfx event")
-                print(wrapper)
                 self.world_object.remove_event_handler(wrapper, t)
             else:
                 feature = getattr(self, f"_{t}")
@@ -348,24 +371,23 @@ class Graphic:
         else:
             return rval
 
-    def __eq__(self, other):
-        # This is necessary because we use Graphics as weakref proxies
-        if not isinstance(other, Graphic):
-            raise TypeError("`==` operator is only valid between two Graphics")
-
-        if self._fpl_address == other._fpl_address:
-            return True
-
-        return False
-
-    def _fpl_cleanup(self):
+    def _fpl_prepare_del(self):
         """
         Cleans up the graphic in preparation for __del__(), such as removing event handlers from
         plot renderer, feature event handlers, etc.
 
         Optionally implemented in subclasses
         """
-        # remove event handlers
+        # remove axes if added to this graphic
+        if self._axes is not None:
+            self._plot_area.scene.remove(self._axes)
+            self._plot_area.remove_animation(self._update_axes)
+            self._axes.world_object.clear()
+
+        # signal that a deletion has been requested
+        self.deleted = True
+
+        # clear event handlers
         self.clear_event_handlers()
 
         # clear any attached event handlers and animation functions
@@ -394,13 +416,10 @@ class Graphic:
 
         self.world_object._event_handlers.clear()
 
-        for n in self._features:
-            fea = getattr(self, f"_{n}")
-            fea.clear_event_handlers()
-
     def __del__(self):
-        self.deleted = True
-        del WORLD_OBJECTS[self._fpl_address]
+        # remove world object if created
+        # world object does not exist if an exception was raised during __init__ which is why this check exists
+        WORLD_OBJECTS.pop(hex(id(self)), None)
 
     def rotate(self, alpha: float, axis: Literal["x", "y", "z"] = "y"):
         """Rotate the Graphic with respect to the world.
@@ -423,3 +442,39 @@ class Graphic:
                 f"`axis` must be either `x`, `y`, or `z`. `{axis}` provided instead!"
             )
         self.rotation = la.quat_mul(rot, self.rotation)
+
+    @property
+    def axes(self) -> Axes:
+        return self._axes
+
+    def add_axes(self):
+        """Add axes onto this Graphic"""
+        if self._axes is not None:
+            raise AttributeError("Axes already added onto this graphic")
+
+        self._axes = Axes(self._plot_area, offset=self.offset, grids=False)
+        self._axes.world_object.local.rotation = self.world_object.local.rotation
+
+        self._plot_area.scene.add(self.axes.world_object)
+        self._axes.update_using_bbox(self.world_object.get_world_bounding_box())
+
+    @property
+    def right_click_menu(self):
+        return self._right_click_menu
+
+    @right_click_menu.setter
+    def right_click_menu(self, menu):
+        if not IMGUI:
+            raise ImportError(
+                "imgui is required to set right-click menus:\n"
+                "pip install imgui_bundle"
+            )
+
+        self._right_click_menu = menu
+        menu.owner = self
+
+    def _fpl_request_right_click_menu(self):
+        pass
+
+    def _fpl_close_right_click_menu(self):
+        pass
