@@ -2,6 +2,7 @@ from ._positions_graphics import (
     VertexColors,
     UniformColor,
     UniformSize,
+    SizeSpace,
     Thickness,
     VertexPositions,
     PointsSizesFeature,
@@ -14,7 +15,6 @@ from ._image import (
     ImageVmax,
     ImageInterpolation,
     ImageCmapInterpolation,
-    WGPU_MAX_TEXTURE_SIZE,
 )
 from ._base import (
     GraphicFeature,
@@ -31,7 +31,11 @@ from ._text import (
     TextOutlineThickness,
 )
 
-from ._selection_features import LinearSelectionFeature, LinearRegionSelectionFeature
+from ._selection_features import (
+    LinearSelectionFeature,
+    LinearRegionSelectionFeature,
+    RectangleSelectionFeature,
+)
 from ._common import Name, Offset, Rotation, Visible, Deleted
 
 
@@ -39,6 +43,7 @@ __all__ = [
     "VertexColors",
     "UniformColor",
     "UniformSize",
+    "SizeSpace",
     "Thickness",
     "VertexPositions",
     "PointsSizesFeature",
@@ -56,6 +61,7 @@ __all__ = [
     "TextOutlineThickness",
     "LinearSelectionFeature",
     "LinearRegionSelectionFeature",
+    "RectangleSelectionFeature",
     "Name",
     "Offset",
     "Rotation",
