@@ -182,6 +182,27 @@ class UniformSize(GraphicFeature):
         self._call_event_handlers(event)
 
 
+# manages the coordinate space for scatter/line
+class SizeSpace(GraphicFeature):
+    def __init__(self, value: str):
+        self._value = value
+        super().__init__()
+
+    @property
+    def value(self) -> str:
+        return self._value
+
+    def set_value(self, graphic, value: str):
+        if "Line" in graphic.world_object.material.__class__.__name__:
+            graphic.world_object.material.thickness_space = value
+        else:
+            graphic.world_object.material.size_space = value
+        self._value = value
+
+        event = FeatureEvent(type="size_space", info={"value": value})
+        self._call_event_handlers(event)
+
+
 class VertexPositions(BufferManager):
     """
     +----------+----------------------------------------------------------+------------------------------------------------------------------------------------------+
