@@ -1,5 +1,4 @@
 from typing import *
-import weakref
 
 import numpy as np
 
@@ -8,7 +7,7 @@ import pygfx
 from ..utils import parse_cmap_values
 from ._collection_base import CollectionIndexer, GraphicCollection, CollectionFeature
 from .line import LineGraphic
-from .selectors import LinearRegionSelector, LinearSelector
+from .selectors import LinearRegionSelector, LinearSelector, RectangleSelector
 
 
 class _LineCollectionProperties:
@@ -378,7 +377,7 @@ class LineCollection(GraphicCollection, _LineCollectionProperties):
             size=size,
             center=center,
             axis=axis,
-            parent=weakref.proxy(self),
+            parent=self,
             **kwargs,
         )
 
@@ -387,7 +386,7 @@ class LineCollection(GraphicCollection, _LineCollectionProperties):
         # place selector above this graphic
         selector.offset = selector.offset + (0.0, 0.0, self.offset[-1] + 1)
 
-        return weakref.proxy(selector)
+        return selector
 
     def add_linear_region_selector(
         self,
@@ -435,7 +434,7 @@ class LineCollection(GraphicCollection, _LineCollectionProperties):
             size=size,
             center=center,
             axis=axis,
-            parent=weakref.proxy(self),
+            parent=self,
             **kwargs,
         )
 
@@ -446,7 +445,48 @@ class LineCollection(GraphicCollection, _LineCollectionProperties):
 
         # PlotArea manages this for garbage collection etc. just like all other Graphics
         # so we should only work with a proxy on the user-end
-        return weakref.proxy(selector)
+        return selector
+
+    def add_rectangle_selector(
+        self,
+        selection: tuple[float, float, float, float] = None,
+        **kwargs,
+    ) -> RectangleSelector:
+        """
+        Add a :class:`.RectangleSelector`. Selectors are just ``Graphic`` objects, so you can manage,
+        remove, or delete them from a plot area just like any other ``Graphic``.
+
+        Parameters
+        ----------
+        selection: (float, float, float, float), optional
+            initial (xmin, xmax, ymin, ymax) of the selection
+        """
+        bbox = self.world_object.get_world_bounding_box()
+
+        xdata = np.array(self.data[:, 0])
+        xmin, xmax = (np.nanmin(xdata), np.nanmax(xdata))
+        value_25px = (xmax - xmin) / 4
+
+        ydata = np.array(self.data[:, 1])
+        ymin = np.floor(ydata.min()).astype(int)
+
+        ymax = np.ptp(bbox[:, 1])
+
+        if selection is None:
+            selection = (xmin, value_25px, ymin, ymax)
+
+        limits = (xmin, xmax, ymin - (ymax * 1.5 - ymax), ymax * 1.5)
+
+        selector = RectangleSelector(
+            selection=selection,
+            limits=limits,
+            parent=self,
+            **kwargs,
+        )
+
+        self._plot_area.add_graphic(selector, center=False)
+
+        return selector
 
     def _get_linear_selector_init_args(self, axis, padding):
         # use bbox to get size and center
