@@ -11,7 +11,7 @@ from warnings import warn
 
 import pygfx
 
-from wgpu.gui import WgpuCanvasBase
+from rendercanvas import BaseRenderCanvas
 
 from ._video_writer import VideoWriterAV
 from ._utils import make_canvas_and_renderer, create_controller, create_camera
@@ -41,7 +41,7 @@ class Figure:
             | Iterable[Iterable[str]]
         ) = None,
         controllers: pygfx.Controller | Iterable[Iterable[pygfx.Controller]] = None,
-        canvas: str | WgpuCanvasBase | pygfx.Texture = None,
+        canvas: str | BaseRenderCanvas | pygfx.Texture = None,
         renderer: pygfx.WgpuRenderer = None,
         size: tuple[int, int] = (500, 300),
         names: list | np.ndarray = None,
@@ -84,8 +84,8 @@ class Figure:
             plot/subplot. Other controller kwargs, i.e. ``controller_types`` and ``controller_ids`` are ignored if
             ``controllers`` are provided.
 
-        canvas: WgpuCanvas, optional
-            Canvas for drawing
+        canvas: str, BaseRenderCanvas, pygfx.Texture
+            Canvas to draw the figure onto, usually auto-selected based on running environment.
 
         renderer: pygfx.Renderer, optional
             pygfx renderer instance
@@ -109,7 +109,9 @@ class Figure:
         else:
             subplot_names = None
 
-        canvas, renderer = make_canvas_and_renderer(canvas, renderer)
+        canvas, renderer = make_canvas_and_renderer(
+            canvas, renderer, canvas_kwargs={"size": size}
+        )
 
         if isinstance(cameras, str):
             # create the array representing the views for each subplot in the grid
@@ -322,38 +324,23 @@ class Figure:
 
         self._current_iter = None
 
-        self._starting_size = size
+        self._sidecar = None
 
         self._output = None
 
-        if self.canvas.__class__.__name__ == "JupyterWgpuCanvas":
-            self.recorder = FigureRecorder(self)
-        else:
-            self.recorder = None
-
-    @property
-    def toolbar(self):
-        """ipywidget or QToolbar instance"""
-        return self._output.toolbar
-
-    @property
-    def output(self):
-        """ipywidget or QWidget that contains this plot"""
-        return self._output
-
     @property
     def shape(self) -> tuple[int, int]:
         """[n_rows, n_cols]"""
         return self._shape
 
     @property
-    def canvas(self) -> WgpuCanvasBase:
-        """The canvas associated to this Figure"""
+    def canvas(self) -> BaseRenderCanvas:
+        """The canvas this Figure is drawn onto"""
         return self._canvas
 
     @property
     def renderer(self) -> pygfx.WgpuRenderer:
-        """The renderer associated to this Figure"""
+        """The renderer that renders this Figure"""
         return self._renderer
 
     @property
@@ -390,7 +377,7 @@ class Figure:
         else:
             return self._subplots[index[0], index[1]]
 
-    def render(self):
+    def render(self, draw=True):
         # call the animation functions before render
         self._call_animate_functions(self._animate_funcs_pre)
 
@@ -398,7 +385,8 @@ class Figure:
             subplot.render()
 
         self.renderer.flush()
-        self.canvas.request_draw()
+        if draw:
+            self.canvas.request_draw()
 
         # call post-render animate functions
         self._call_animate_functions(self._animate_funcs_post)
@@ -406,19 +394,16 @@ class Figure:
     def start_render(self):
         """start render cycle"""
         self.canvas.request_draw(self.render)
-        self.canvas.set_logical_size(*self._starting_size)
 
     def show(
         self,
         autoscale: bool = True,
         maintain_aspect: bool = None,
-        toolbar: bool = True,
         sidecar: bool = False,
         sidecar_kwargs: dict = None,
-        add_widgets: list = None,
     ):
         """
-        Begins the rendering event loop and shows the plot in the desired output context (jupyter, qt or glfw).
+        Begins the rendering event loop and shows the Figure, returns the canvas
 
         Parameters
         ----------
@@ -428,29 +413,22 @@ class Figure:
         maintain_aspect: bool, default ``True``
             maintain aspect ratio
 
-        toolbar: bool, default ``True``
-            show toolbar
-
         sidecar: bool, default ``True``
-            display plot in a ``jupyterlab-sidecar``, only for jupyter output context
+            display plot in a ``jupyterlab-sidecar``, only in jupyter
 
         sidecar_kwargs: dict, default ``None``
             kwargs for sidecar instance to display plot
             i.e. title, layout
 
-        add_widgets: list of widgets
-            a list of ipywidgets or QWidget that are vertically stacked below the plot
-
         Returns
         -------
-        OutputContext
-            In jupyter, it will display the plot in the output cell or sidecar
-
-            In Qt, it will display the Plot, toolbar, etc. as stacked widget, use `Plot.widget` to access it.
+        BaseRenderCanvas
+            In Qt or GLFW, the canvas window containing the Figure will be shown.
+            In jupyter, it will display the plot in the output cell or sidecar.
         """
 
-        # show was already called, return existing output context
-        if self._output is not None:
+        # show was already called, return canvas
+        if self._output:
             return self._output
 
         self.start_render()
@@ -458,9 +436,6 @@ class Figure:
         if sidecar_kwargs is None:
             sidecar_kwargs = dict()
 
-        if add_widgets is None:
-            add_widgets = list()
-
         # flip y-axis if ImageGraphics are present
         for subplot in self:
             for g in subplot.graphics:
@@ -474,37 +449,50 @@ class Figure:
                     _maintain_aspect = subplot.camera.maintain_aspect
                 else:
                     _maintain_aspect = maintain_aspect
-                subplot.auto_scale(maintain_aspect=_maintain_aspect, zoom=0.95)
+                subplot.auto_scale(maintain_aspect=maintain_aspect)
 
-        # return the appropriate OutputContext based on the current canvas
-        if self.canvas.__class__.__name__ == "JupyterWgpuCanvas":
-            from .output.jupyter_output import (
-                JupyterOutputContext,
-            )  # noqa - inline import
+        # parse based on canvas type
+        if self.canvas.__class__.__name__ == "JupyterRenderCanvas":
+            if sidecar:
+                from sidecar import Sidecar
+                from IPython.display import display
 
-            self._output = JupyterOutputContext(
-                frame=self,
-                make_toolbar=toolbar,
-                use_sidecar=sidecar,
-                sidecar_kwargs=sidecar_kwargs,
-                add_widgets=add_widgets,
-            )
+                self._sidecar = Sidecar(**sidecar_kwargs)
+                self._output = self.canvas
+                with self._sidecar:
+                    return display(self.canvas)
+            self._output = self.canvas
+            return self._output
 
-        elif self.canvas.__class__.__name__ == "QWgpuCanvas":
-            from .output.qt_output import QOutputContext  # noqa - inline import
+        elif self.canvas.__class__.__name__ == "QRenderCanvas":
+            self._output = self.canvas
+            self._output.show()
+            return self.canvas
 
-            self._output = QOutputContext(
-                frame=self, make_toolbar=toolbar, add_widgets=add_widgets
-            )
+        elif self.canvas.__class__.__name__ == "OffscreenRenderCanvas":
+            # for test and docs gallery screenshots
+            for subplot in self:
+                subplot.set_viewport_rect()
+                subplot.axes.update_using_camera()
 
-        else:  # assume GLFW, the output context is just the canvas
+                # render call is blocking only on github actions for some reason,
+                # but not for rtd build, this is a workaround
+                # for CI tests, the render call works if it's in test_examples
+                # but it is necessary for the gallery images too so that's why this check is here
+                if "RTD_BUILD" in os.environ.keys():
+                    if os.environ["RTD_BUILD"] == "1":
+                        self.render()
+
+        else:  # assume GLFW
             self._output = self.canvas
 
-        # return the output context, this call is required for jupyter but not for Qt
+        # return the canvas
         return self._output
 
     def close(self):
-        self.output.close()
+        self._output.close()
+        if self._sidecar:
+            self._sidecar.close()
 
     def _call_animate_functions(self, funcs: list[callable]):
         for fn in funcs:
@@ -580,6 +568,37 @@ class Figure:
         for subplot in self:
             subplot.clear()
 
+    def export_numpy(self, rgb: bool = False) -> np.ndarray:
+        """
+        Export a snapshot of the Figure as numpy array.
+
+        Parameters
+        ----------
+        rgb: bool, default ``False``
+            if True, use alpha blending to return an RGB image.
+            if False, returns an RGBA array
+
+        Returns
+        -------
+        np.ndarray
+            [n_rows, n_cols, 3] for RGB or [n_rows, n_cols, 4] for RGBA
+        """
+        snapshot = self.renderer.snapshot()
+
+        if rgb:
+            bg = np.zeros(snapshot.shape).astype(np.uint8)
+            bg[:, :, -1] = 255
+
+            img_alpha = snapshot[..., -1] / 255
+
+            rgb = snapshot[..., :-1] * img_alpha[..., None] + bg[..., :-1] * np.ones(
+                img_alpha.shape
+            )[..., None] * (1 - img_alpha[..., None])
+
+            return rgb.astype(np.uint8)
+
+        return snapshot
+
     def export(self, uri: str | Path | bytes, **kwargs):
         """
         Use ``imageio`` for writing the current Figure to a file, or return a byte string.
@@ -605,27 +624,40 @@ class Figure:
                 "conda install -c conda-forge imageio\n"
             )
         else:
-            snapshot = self.renderer.snapshot()
-            remove_alpha = True
-
             # image formats that support alpha channel:
             # https://en.wikipedia.org/wiki/Alpha_compositing#Image_formats_supporting_alpha_channels
             alpha_support = [".png", ".exr", ".tiff", ".tif", ".gif", ".jxl", ".svg"]
 
-            if isinstance(uri, str):
-                if any([uri.endswith(ext) for ext in alpha_support]):
-                    remove_alpha = False
+            uri = Path(uri)
 
-            elif isinstance(uri, Path):
-                if uri.suffix in alpha_support:
-                    remove_alpha = False
+            if uri.suffix in alpha_support:
+                rgb = False
+            else:
+                rgb = True
 
-            if remove_alpha:
-                # remove alpha channel if it's not supported
-                snapshot = snapshot[..., :-1].shape
+            snapshot = self.export_numpy(rgb=rgb)
 
             return iio.imwrite(uri, snapshot, **kwargs)
 
+    def open_popup(self, *args, **kwargs):
+        warn("popups only supported by ImguiFigure")
+
+    def get_pygfx_render_area(self, *args) -> tuple[int, int, int, int]:
+        """
+        Fet rect for the portion of the canvas that the pygfx renderer draws to,
+        i.e. non-imgui, part of canvas
+
+        Returns
+        -------
+        tuple[int, int, int, int]
+            x_pos, y_pos, width, height
+
+        """
+
+        width, height = self.canvas.get_logical_size()
+
+        return 0, 0, width, height
+
     def _get_iterator(self):
         return product(range(self.shape[0]), range(self.shape[1]))
 
@@ -653,157 +685,3 @@ class Figure:
             f"\t{newline.join(subplot.__str__() for subplot in self)}"
             f"\n"
         )
-
-
-class FigureRecorder:
-    def __init__(self, figure: Figure):
-        self._figure = figure
-        self._video_writer: VideoWriterAV = None
-        self._video_writer_queue = Queue()
-        self._record_fps = 25
-        self._record_timer = 0
-        self._record_start_time = 0
-
-    def _record(self):
-        """
-        Sends frame to VideoWriter through video writer queue
-        """
-        # current time
-        t = time()
-
-        # put frame in queue only if enough time as passed according to the desired framerate
-        # otherwise it tries to record EVERY frame on every rendering cycle, which just blocks the rendering
-        if t - self._record_timer < (1 / self._record_fps):
-            return
-
-        # reset timer
-        self._record_timer = t
-
-        if self._video_writer is not None:
-            ss = self._figure.canvas.snapshot()
-            # exclude alpha channel
-            self._video_writer_queue.put(ss.data[..., :-1])
-
-    def start(
-        self,
-        path: str | Path,
-        fps: int = 25,
-        codec: str = "mpeg4",
-        pixel_format: str = "yuv420p",
-        options: dict = None,
-    ):
-        """
-        Start a recording, experimental. Call ``record_end()`` to end a recording.
-        Note: playback duration does not exactly match recording duration.
-
-        Requires PyAV: https://github.com/PyAV-Org/PyAV
-
-        **Do not resize canvas during a recording, the width and height must remain constant!**
-
-        Parameters
-        ----------
-        path: str or Path
-            path to save the recording
-
-        fps: int, default ``25``
-            framerate, do not use > 25 within jupyter
-
-        codec: str, default "mpeg4"
-            codec to use, see ``ffmpeg`` list: https://www.ffmpeg.org/ffmpeg-codecs.html .
-            In general, ``"mpeg4"`` should work on most systems. ``"libx264"`` is a
-            better option if you have it installed.
-
-        pixel_format: str, default "yuv420p"
-            pixel format
-
-        options: dict, optional
-            Codec options. For example, if using ``"mpeg4"`` you can use ``{"q:v": "20"}`` to set the quality between
-            1-31, where "1" is highest and "31" is lowest. If using ``"libx264"``` you can use ``{"crf": "30"}`` where
-            the "crf" value is between "0" (highest quality) and "50" (lowest quality). See ``ffmpeg`` docs for more
-            info on codec options
-
-        Examples
-        --------
-
-        With ``"mpeg4"``
-
-        .. code-block:: python
-
-            # start recording video
-            figure.recorder.start("./video.mp4", options={"q:v": "20"}
-
-            # do stuff like interacting with the plot, change things, etc.
-
-            # end recording
-            figure.recorder.stop()
-
-        With ``"libx264"``
-
-        .. code-block:: python
-
-            # start recording video
-            figure.recorder.start("./vid_x264.mp4", codec="libx264", options={"crf": "25"})
-
-            # do stuff like interacting with the plot, change things, etc.
-
-            # end recording
-            figure.recorder.stop()
-
-        """
-
-        if Path(path).exists():
-            raise FileExistsError(f"File already exists at given path: {path}")
-
-        # queue for sending frames to VideoWriterAV process
-        self._video_writer_queue = Queue()
-
-        # snapshot to get canvas width height
-        ss = self._figure.canvas.snapshot()
-
-        # writer process
-        self._video_writer = VideoWriterAV(
-            path=str(path),
-            queue=self._video_writer_queue,
-            fps=int(fps),
-            width=ss.width,
-            height=ss.height,
-            codec=codec,
-            pixel_format=pixel_format,
-            options=options,
-        )
-
-        # start writer process
-        self._video_writer.start()
-
-        # 1.3 seems to work well to reduce that difference between playback time and recording time
-        # will properly investigate later
-        self._record_fps = fps * 1.3
-        self._record_start_time = time()
-
-        # record timer used to maintain desired framerate
-        self._record_timer = time()
-
-        self._figure.add_animations(self._record)
-
-    def stop(self) -> float:
-        """
-        End a current recording. Returns the real duration of the recording
-
-        Returns
-        -------
-        float
-            recording duration
-        """
-
-        # tell video writer that recording has finished
-        self._video_writer_queue.put(None)
-
-        # wait for writer to finish
-        self._video_writer.join(timeout=5)
-
-        self._video_writer = None
-
-        # so self._record() is no longer called on every render cycle
-        self._figure.remove_animation(self._record)
-
-        return time() - self._record_start_time
