@@ -1,96 +1,30 @@
 from inspect import getfullargspec
-from sys import getrefcount
-from typing import TypeAlias, Literal, Union
-import weakref
+from typing import Literal, Union
 from warnings import warn
 
 import numpy as np
 
 import pygfx
 from pylinalg import vec_transform, vec_unproject
-from wgpu.gui import WgpuCanvasBase
+from rendercanvas import BaseRenderCanvas
 
 from ._utils import create_controller
 from ..graphics._base import Graphic
-from ..graphics._collection_base import GraphicCollection
 from ..graphics.selectors._base_selector import BaseSelector
 from ..legends import Legend
 
 
-HexStr: TypeAlias = str
-
-
-class References:
-    """
-    This is the only place where the real graphic objects are stored. Everywhere else gets a proxy.
-    """
-
-    _graphics: dict[HexStr, Graphic] = dict()
-    _selectors: dict[HexStr, BaseSelector] = dict()
-    _legends: dict[HexStr, Legend] = dict()
-
-    def add(self, graphic: Graphic | BaseSelector | Legend):
-        """Adds the real graphic to the dict"""
-        addr = graphic._fpl_address
-
-        if isinstance(graphic, BaseSelector):
-            self._selectors[addr] = graphic
-
-        elif isinstance(graphic, Legend):
-            self._legends[addr] = graphic
-
-        elif isinstance(graphic, Graphic):
-            self._graphics[addr] = graphic
-
-        else:
-            raise TypeError("Can only add Graphic, Selector or Legend types")
-
-    def remove(self, address):
-        if address in self._graphics.keys():
-            del self._graphics[address]
-        elif address in self._selectors.keys():
-            del self._selectors[address]
-        elif address in self._legends.keys():
-            del self._legends[address]
-        else:
-            raise KeyError(f"graphic with address not found: {address}")
-
-    def get_proxies(self, refs: list[HexStr]) -> tuple[weakref.proxy]:
-        proxies = list()
-        for key in refs:
-            if key in self._graphics.keys():
-                proxies.append(weakref.proxy(self._graphics[key]))
-
-            elif key in self._selectors.keys():
-                proxies.append(weakref.proxy(self._selectors[key]))
-
-            elif key in self._legends.keys():
-                proxies.append(weakref.proxy(self._legends[key]))
-
-            else:
-                raise KeyError(f"graphic object with address not found: {key}")
-
-        return tuple(proxies)
-
-    def get_refcounts(self) -> dict[HexStr:int]:
-        counts = dict()
-
-        for item in (self._graphics, self._selectors, self._legends):
-            for k in item.keys():
-                counts[(k, item[k].name, item[k].__class__.__name__)] = getrefcount(
-                    item[k]
-                )
-
-        return counts
-
-
-REFERENCES = References()
+try:
+    get_ipython()
+except NameError:
+    IS_IPYTHON = False
+    IPYTHON = None
+else:
+    IS_IPYTHON = True
+    IPYTHON = get_ipython()
 
 
 class PlotArea:
-    def get_refcounts(self):
-        return REFERENCES.get_refcounts()
-
     def __init__(
         self,
         parent: Union["PlotArea", "Figure"],
@@ -98,7 +32,7 @@ class PlotArea:
         camera: pygfx.PerspectiveCamera,
         controller: pygfx.Controller,
         scene: pygfx.Scene,
-        canvas: WgpuCanvasBase,
+        canvas: BaseRenderCanvas,
         renderer: pygfx.WgpuRenderer,
         name: str = None,
     ):
@@ -124,7 +58,7 @@ class PlotArea:
         scene: pygfx.Scene
             represents the root of a scene graph, will be viewed by the given ``camera``
 
-        canvas: WgpuCanvas
+        canvas: BaseRenderCanvas
             provides surface on which a scene will be rendered
 
         renderer: pygfx.WgpuRenderer
@@ -158,22 +92,49 @@ class PlotArea:
 
         # list of hex id strings for all graphics managed by this PlotArea
         # the real Graphic instances are managed by REFERENCES
-        self._graphics: list[HexStr] = list()
+        self._graphics: list[Graphic] = list()
 
         # selectors are in their own list so they can be excluded from scene bbox calculations
         # managed similar to GRAPHICS for garbage collection etc.
-        self._selectors: list[HexStr] = list()
+        self._selectors: list[BaseSelector] = list()
 
         # legends, managed just like other graphics as explained above
-        self._legends: list[HexStr] = list()
+        self._legends: list[Legend] = list()
+
+        # keep all graphics in a separate group, makes bbox calculations etc. easier
+        # this is the "real scene" excluding axes, selection tools etc.
+        self._fpl_graphics_scene = pygfx.Group()
+        self.scene.add(self._fpl_graphics_scene)
 
         self._name = name
 
         # need to think about how to deal with children better
         self.children = list()
 
+        self._background_material = pygfx.BackgroundMaterial(
+            (0.0, 0.0, 0.0, 1.0),
+            (0.0, 0.0, 0.0, 1.0),
+            (0.0, 0.0, 0.0, 1.0),
+            (0.0, 0.0, 0.0, 1.0),
+        )
+        self._background = pygfx.Background(None, self._background_material)
+        self.scene.add(self._background)
+
         self.set_viewport_rect()
 
+    def get_figure(self, obj=None):
+        """Get Figure instance that contains this plot area"""
+        if obj is None:
+            obj = self
+
+        if obj.parent.__class__.__name__.endswith("Figure"):
+            return obj.parent
+        else:
+            if obj.parent is None:
+                raise RecursionError
+
+            return self.get_figure(obj=obj.parent)
+
     # several read-only properties
     @property
     def parent(self):
@@ -191,7 +152,7 @@ class PlotArea:
         return self._scene
 
     @property
-    def canvas(self) -> WgpuCanvasBase:
+    def canvas(self) -> BaseRenderCanvas:
         """Canvas associated to the plot area"""
         return self._canvas
 
@@ -266,7 +227,7 @@ class PlotArea:
         # TODO: monkeypatch until we figure out a better
         #  pygfx plans on refactoring viewports anyways
         if self.parent is not None:
-            if self.parent.__class__.__name__ == "Figure":
+            if self.parent.__class__.__name__.endswith("Figure"):
                 for subplot in self.parent:
                     if subplot.camera in cameras_list:
                         new_controller.register_events(subplot.viewport)
@@ -276,18 +237,18 @@ class PlotArea:
 
     @property
     def graphics(self) -> tuple[Graphic, ...]:
-        """Graphics in the plot area. Always returns a proxy to the Graphic instances."""
-        return REFERENCES.get_proxies(self._graphics)
+        """Graphics in the plot area."""
+        return tuple(self._graphics)
 
     @property
     def selectors(self) -> tuple[BaseSelector, ...]:
-        """Selectors in the plot area. Always returns a proxy to the Graphic instances."""
-        return REFERENCES.get_proxies(self._selectors)
+        """Selectors in the plot area."""
+        return tuple(self._selectors)
 
     @property
     def legends(self) -> tuple[Legend, ...]:
         """Legends in the plot area."""
-        return REFERENCES.get_proxies(self._legends)
+        return tuple(self._legends)
 
     @property
     def objects(self) -> tuple[Graphic | BaseSelector | Legend, ...]:
@@ -308,6 +269,21 @@ class PlotArea:
             raise TypeError("PlotArea `name` must be of type <str>")
         self._name = name
 
+    @property
+    def background_color(self) -> tuple[pygfx.Color, ...]:
+        """background colors, (top left, top right, bottom right, bottom left)"""
+        return (
+            self._background_material.color_top_left,
+            self._background_material.color_top_right,
+            self._background_material.color_bottom_right,
+            self._background_material.color_bottom_left,
+        )
+
+    @background_color.setter
+    def background_color(self, colors: str | tuple[float]):
+        """1, 2, or 4 colors, each color must be acceptable by pygfx.Color"""
+        self._background_material.set_colors(*colors)
+
     def get_rect(self) -> tuple[float, float, float, float]:
         """
         Returns the viewport rect to define the rectangle
@@ -462,7 +438,7 @@ class PlotArea:
 
         if graphic in self:
             # graphic is already in this plot but was removed from the scene, add it back
-            self.scene.add(graphic.world_object)
+            self._fpl_graphics_scene.add(graphic.world_object)
             return
 
         self._add_or_insert_graphic(graphic=graphic, center=center, action="add")
@@ -531,35 +507,28 @@ class PlotArea:
         if graphic.name is not None:  # skip for those that have no name
             self._check_graphic_name_exists(graphic.name)
 
-        addr = graphic._fpl_address
-
         if isinstance(graphic, BaseSelector):
-            addr_list = self._selectors
+            obj_list = self._selectors
+            self.scene.add(graphic.world_object)
 
         elif isinstance(graphic, Legend):
-            addr_list = self._legends
+            obj_list = self._legends
+            self.scene.add(graphic.world_object)
 
         elif isinstance(graphic, Graphic):
-            addr_list = self._graphics
+            obj_list = self._graphics
+            self._fpl_graphics_scene.add(graphic.world_object)
 
         else:
             raise TypeError("graphic must be of type Graphic | BaseSelector | Legend")
 
         if action == "insert":
-            addr_list.insert(index, addr)
+            obj_list.insert(index, graphic)
         elif action == "add":
-            addr_list.append(addr)
+            obj_list.append(graphic)
         else:
             raise ValueError("valid actions are 'insert' | 'add'")
 
-        REFERENCES.add(graphic)
-
-        # now that it's in the dict, just use the weakref
-        graphic = weakref.proxy(graphic)
-
-        # add world object to scene
-        self.scene.add(graphic.world_object)
-
         if center:
             self.center_graphic(graphic)
 
@@ -573,7 +542,7 @@ class PlotArea:
                 f"All graphics within a subplot or plot area must have a unique name."
             )
 
-    def center_graphic(self, graphic: Graphic, zoom: float = 1.35):
+    def center_graphic(self, graphic: Graphic, zoom: float = 1.0):
         """
         Center the camera w.r.t. the passed graphic
 
@@ -582,7 +551,7 @@ class PlotArea:
         graphic: Graphic
             The graphic instance to center on
 
-        zoom: float, default 1.3
+        zoom: float
             zoom the camera after centering
 
         """
@@ -593,23 +562,24 @@ class PlotArea:
         # probably because camera.show_object uses bounding sphere
         self.camera.zoom = zoom
 
-    def center_scene(self, *, zoom: float = 1.35):
+    def center_scene(self, *, zoom: float = 1.0):
         """
         Auto-center the scene, does not scale.
 
         Parameters
         ----------
-        zoom: float, default 1.3
+        zoom: float
             apply a zoom after centering the scene
 
         """
-        if not len(self.scene.children) > 0:
+
+        if not len(self._fpl_graphics_scene.children) > 0:
             return
 
         # scale all cameras associated with this controller
         # else it looks wonky
         for camera in self.controller.cameras:
-            camera.show_object(self.scene)
+            camera.show_object(self._fpl_graphics_scene)
 
             # camera.show_object can cause the camera width and height to increase so apply a zoom to compensate
             # probably because camera.show_object uses bounding sphere
@@ -619,7 +589,7 @@ class PlotArea:
         self,
         *,  # since this is often used as an event handler, don't want to coerce maintain_aspect = True
         maintain_aspect: None | bool = None,
-        zoom: float = 0.8,
+        zoom: float = 0.75,
     ):
         """
         Auto-scale the camera w.r.t to the scene
@@ -630,16 +600,13 @@ class PlotArea:
             Maintain the camera aspect ratio for all dimensions. If ``None``, the aspect is left unchanged.
             if ``False`` the camera is scaled to the bounding box of the current scene.
 
-        zoom: float, default 0.8
-            zoom value for the camera after auto-scaling, if zoom = 1.0 then the graphics
-            in the scene will fill the entire canvas.
+        zoom: float
+            zoom value for the camera after auto-scaling
+
         """
-        if not len(self.scene.children) > 0:
+
+        if not len(self._fpl_graphics_scene.children) > 0:
             return
-        # hacky workaround for now until we decide if we want to put selectors in their own scene
-        # remove all selectors from a scene to calculate scene bbox
-        for selector in self.selectors:
-            self.scene.remove(selector.world_object)
 
         self.center_scene()
 
@@ -650,8 +617,10 @@ class PlotArea:
         for camera in self.controller.cameras:
             camera.maintain_aspect = maintain_aspect
 
-        if len(self.scene.children) > 0:
-            width, height, depth = np.ptp(self.scene.get_world_bounding_box(), axis=0)
+        if len(self._fpl_graphics_scene.children) > 0:
+            width, height, depth = np.ptp(
+                self._fpl_graphics_scene.get_world_bounding_box(), axis=0
+            )
         else:
             width, height, depth = (1, 1, 1)
 
@@ -661,9 +630,6 @@ class PlotArea:
         if height < 0.01:
             height = 1
 
-        for selector in self.selectors:
-            self.scene.add(selector.world_object)
-
         # scale all cameras associated with this controller else it looks wonky
         for camera in self.controller.cameras:
             camera.width = width
@@ -684,7 +650,11 @@ class PlotArea:
 
         """
 
-        self.scene.remove(graphic.world_object)
+        if isinstance(graphic, (BaseSelector, Legend)):
+            self.scene.remove(graphic.world_object)
+
+        elif isinstance(graphic, Graphic):
+            self._fpl_graphics_scene.remove(graphic.world_object)
 
     def delete_graphic(self, graphic: Graphic):
         """
@@ -696,29 +666,38 @@ class PlotArea:
             The graphic to delete
 
         """
-        # TODO: proper gc of selectors, RAM is freed for regular graphics but not selectors
-        # TODO: references to selectors must be lingering somewhere
-        # TODO: update March 2024, I think selectors are gc properly, should check
-        # get memory address
-        address = graphic._fpl_address
-
         if graphic not in self:
             raise KeyError(f"Graphic not found in plot area: {graphic}")
 
-        # check which type it is
-        for l in [self._graphics, self._selectors, self._legends]:
-            if address in l:
-                l.remove(address)
-                break
+        if isinstance(graphic, BaseSelector):
+            self._selectors.remove(graphic)
+
+        elif isinstance(graphic, Legend):
+            self._legends.remove(graphic)
+
+        elif isinstance(graphic, Graphic):
+            self._graphics.remove(graphic)
 
         # remove from scene if necessary
         if graphic.world_object in self.scene.children:
             self.scene.remove(graphic.world_object)
 
-        # cleanup
-        graphic._fpl_cleanup()
+        elif graphic.world_object in self._fpl_graphics_scene.children:
+            self._fpl_graphics_scene.remove(graphic.world_object)
 
-        REFERENCES.remove(address)
+        # cleanup
+        graphic._fpl_prepare_del()
+
+        if IS_IPYTHON:
+            # remove any references that ipython might have made
+            # check both namespaces
+            for namespace in [IPYTHON.user_ns, IPYTHON.user_ns_hidden]:
+                # find the reference
+                for ref, obj in namespace.items():
+                    if graphic is obj:
+                        # we found the reference, remove from ipython
+                        IPYTHON.del_var(ref)
+                        break
 
     def clear(self):
         """
